/*++

Copyright (c)  1999 - 2002 Intel Corporation. All rights reserved
This software and associated documentation (if any) is furnished
under a license and may only be used or copied in accordance
with the terms of the license. Except as permitted by such
license, no part of this software or documentation may be
reproduced, stored in a retrieval system, or transmitted in any
form or by any means without the express written consent of
Intel Corporation.


Module Name:

  DeviceIo.h
  
Abstract:
  Private Data definition for Device IO driver

--*/

#ifndef _DEVICE_IO_H
#define _DEVICE_IO_H

#include "Tiano.h"
#include "EfiDriverLib.h"
#include "pci22.h"

//
// Driver Consumed Protocol Prototypes
//
#include EFI_PROTOCOL_DEFINITION (PciRootBridgeIo)
#include EFI_PROTOCOL_DEFINITION (DevicePath)

//
// Driver Produced Protocol Prototypes
//
#include EFI_PROTOCOL_DEFINITION (DeviceIo)


#define DEVICE_IO_PRIVATE_DATA_SIGNATURE   EFI_SIGNATURE_32('d','e','v','I')

typedef struct {
  UINTN                             Signature;
  EFI_DEVICE_IO_PROTOCOL            DeviceIo;
  EFI_PCI_ROOT_BRIDGE_IO_PROTOCOL   *PciRootBridgeIo;
  EFI_DEVICE_PATH_PROTOCOL          *DevicePath;
  UINT16                            PrimaryBus;
  UINT16                            SubordinateBus;
} DEVICE_IO_PRIVATE_DATA;

#define DEVICE_IO_PRIVATE_DATA_FROM_THIS(a) CR(a, DEVICE_IO_PRIVATE_DATA, DeviceIo, DEVICE_IO_PRIVATE_DATA_SIGNATURE)

#define NT32_MAX_PHISICAL_ADDRESS 0xffffffff

#endif


EFI_STATUS
DeviceIoMemRead (
  IN EFI_DEVICE_IO_PROTOCOL *This,
  IN EFI_IO_WIDTH           Width,
  IN UINT64                 Address,
  IN UINTN                  Count,
  IN OUT VOID               *Buffer
  );

EFI_STATUS
DeviceIoMemWrite (
  IN EFI_DEVICE_IO_PROTOCOL *This,
  IN EFI_IO_WIDTH           Width,
  IN UINT64                 Address,
  IN UINTN                  Count,
  IN OUT VOID               *Buffer
  );

EFI_STATUS
DeviceIoIoRead (
  IN EFI_DEVICE_IO_PROTOCOL *This,
  IN EFI_IO_WIDTH           Width,
  IN UINT64                 Address,
  IN UINTN                  Count,
  IN OUT VOID               *Buffer
  );

EFI_STATUS
DeviceIoIoWrite (
  IN EFI_DEVICE_IO_PROTOCOL *This,
  IN EFI_IO_WIDTH           Width,
  IN UINT64                 Address,
  IN UINTN                  Count,
  IN OUT VOID               *Buffer
  );

EFI_STATUS
DeviceIoPciRead (
  IN EFI_DEVICE_IO_PROTOCOL *This,
  IN EFI_IO_WIDTH           Width,
  IN UINT64                 Address,
  IN UINTN                  Count,
  IN OUT VOID               *Buffer
  );

EFI_STATUS
DeviceIoPciWrite (
  IN EFI_DEVICE_IO_PROTOCOL *This,
  IN EFI_IO_WIDTH           Width,
  IN UINT64                 Address,
  IN UINTN                  Count,
  IN OUT VOID               *Buffer
  );

EFI_STATUS
DeviceIoPciDevicePath (
  IN EFI_DEVICE_IO_PROTOCOL        *This,
  IN UINT64                        Address,
  IN OUT EFI_DEVICE_PATH_PROTOCOL  **PciDevicePath
  );

EFI_STATUS
DeviceIoMap (
  IN EFI_DEVICE_IO_PROTOCOL   *This,
  IN EFI_IO_OPERATION_TYPE    Operation,
  IN EFI_PHYSICAL_ADDRESS     *HostAddress,
  IN OUT UINTN                *NumberOfBytes,
  OUT EFI_PHYSICAL_ADDRESS    *DeviceAddress,
  OUT VOID                    **Mapping
  );

EFI_STATUS
DeviceIoUnmap (
  IN EFI_DEVICE_IO_PROTOCOL   *This,
  IN VOID                     *Mapping
  );

EFI_STATUS
DeviceIoAllocateBuffer (
  IN EFI_DEVICE_IO_PROTOCOL    *This,
  IN EFI_ALLOCATE_TYPE         Type,
  IN EFI_MEMORY_TYPE           MemoryType,
  IN UINTN                     Pages,
  IN OUT EFI_PHYSICAL_ADDRESS  *HostAddress
  );

EFI_STATUS
DeviceIoFlush (
  IN EFI_DEVICE_IO_PROTOCOL  *This
  );

EFI_STATUS
DeviceIoFreeBuffer (
  IN EFI_DEVICE_IO_PROTOCOL   *This,
  IN UINTN                    Pages,
  IN EFI_PHYSICAL_ADDRESS     HostAddress
  );
  
EFI_STATUS
DeviceIoConstructor (
  );


